import time
from sqlalchemy import (
    Column,
    Integer,
    String,
    Date,
    BigInteger,
    create_engine,
    MetaData,
)
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy.schema import CreateTable, CreateIndex

Base = declarative_base()


class RawTransfer(Base):
    __tablename__ = "raw_transfer"
    id = Column(Integer, primary_key=True)
    raw = Column(String)
    uid = Column(String(128), index=True)
    on_account = Column(String(32), index=True)
    amount = Column(Integer)
    currency = Column(String(8))
    date = Column(Date)
    type = Column(String(16))
    index = Column(Integer)

    title = Column(String(256))

    balance = Column(Integer)
    balance_currency = Column(String(8))

    from_account = Column(String(32))
    to_account = Column(String(32))

    from_name = Column(String(256))
    to_name = Column(String(256))

    scrape_timestamp = Column(BigInteger, default=lambda: round(time.time() * 1000000))

    def __str__(self):
        return u'{} *{} #{} @{} -"{}" -#{} => +"{}" +#{} [{}.{:02d} {}] ~"{}"'.format(
            self.type,
            self.uid,
            self.on_account,
            self.date,
            self.from_name,
            self.from_account,
            self.to_name,
            self.to_account,
            self.amount // 100,
            self.amount % 100,
            self.currency,
            self.title,
        )

    def __repr__(self):
        return "<Transfer %s>" % (str(self),)


def get_schema(engine):
    schema = ""

    m = MetaData()
    schema += "%s;\n" % (CreateTable(RawTransfer.__table__).compile(engine),)
    for index in RawTransfer.__table__.indexes:
        schema += "%s;\n" % (CreateIndex(index).compile(engine),)

    return schema
